/* Projektortimer */

#include <inttypes.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include <avr/signal.h>
#include <avr/io.h>
#include <avr/eeprom.h>

#define TMC8_CK1024     (_BV(CS02)+_BV(CS00))
#define TMC8_CK256      _BV(CS02)
#define TMC8_CK64       (BV(CS00)+BV(CS01))
#define EIN				0
#define AUS				64
#define	DELAY_LOOPS		250
#define RUN				0
#define SET				1

const unsigned char ziffer[]={0x20,0x6E,0x09,0x0c,0x46,0x14,0x10,0x2E,0x00,0x04};
volatile unsigned char zeit=2;  		// aktueller Zhlerstand
volatile unsigned char led=AUS;     	// 0=ein, 64=aus
volatile unsigned char maxzeit=15;  	// default, falls EEPROM ungltig
volatile unsigned char stelle=0;    	// Displaystelle
volatile unsigned char takt=0;      	// 1/255 Sekundentakt
volatile unsigned char state=0;     	// 0=run, 1=set
volatile unsigned char nKeyPress;   	// Anzahl Tastendrucke
volatile unsigned char nCounter;    	// Schleifenzhler

struct 
{
    unsigned char bPin:1;				// Bit 0 fr das Signal am Pin 0.
    unsigned char bStatus:1;			// Bit 1 fr das Speichern des Status der Taste
										// Die verbleibenden 6 Bits der Variable werden nicht
										// genutzt.
} bPortB;

void toggle_state(void)
{
if (state == RUN) 	state = SET;		// Status umschalten
else 
	{
	cli();
	_EEPUT(0x00,zeit);					// schreibe neuen Wert ins EEPROM
	while bit_is_set(EECR, EEWE)		// warte bis EEPROM bereit 
		{
		}	
	maxzeit = zeit;						// Zeit aktualisieren
	state = RUN;						// Status umschalten
	sei();
	}
}

INTERRUPT(SIG_OVERFLOW0)
{
// char graytab[] = {0,1,-1,0,-1,0,0,1,1,0,0,-1,0,-1,1,0 };  // 1 Flanke je Raste
char graytab[] = {0,0,0,0,-1,0,0,1,1,0,0,-1,0,0,0,0 };  // 2 Flanken je Raste
char graycount=0;
static unsigned char graycode = 0;

PORTB |= _BV(PB5)+_BV(PB6)+_BV(PB7); 	 //alle Stellen aus
switch (stelle)
	{
	case 0:
	// Einer ausgeben
	PORTD = ziffer[zeit/10];
	PORTB &= ~_BV(PB7);
	break;
	case 1:
	// Zehner ausgeben
	PORTD = ziffer[zeit%10];
	PORTB &= ~_BV(PB6);
	break;
	case 2:       
	// LED Status ausgeben
	PORTD=led;
	PORTB &= ~_BV(PB5);
	break;
	}

if ((stelle++) > 2) stelle=0;			// Stellenzhler erhhen bzw. zurcksetzen

/* Zeit zhlen und Impulse ausgeben */
if (state == RUN)
    {
	PORTB |= _BV(PB4);					// Dezimalpunkte aus

	// erste Variante: LED nur an
	led=EIN;
	// zweite Variante: LED nach 0.5s wieder aus
	if (takt>127) led=AUS;

	if ((takt++)==255)
		{
		if (zeit != 0) zeit--;
		takt=0;
		}
	if (zeit == 0) 
		{
		PORTB |= _BV(PB0);				// Impulspin ein
		if (takt>64)
			{
			PORTB &= ~(_BV(PB0));		// Impulspin wieder aus

// dritte Variante: LED als Pulskontrolle
//			led=AUS;

			}
		if (takt==255) zeit = (maxzeit+1);
		}
	}

/* Zeit eingeben */
if (state == SET)
	{
	led=AUS;
	if bit_is_set(PINB,PINB1) graycode |= 0x10;
	if bit_is_set(PINB,PINB2) graycode |= 0x20;
	graycode = (graycode>>2);
	graycount += graytab[graycode];
	zeit += graycount;
	if (zeit < 2) zeit=99;
	if (zeit > 99) zeit=2;
	graycount = 0;
	
	//Blinken der Dezimalpunkte 
	PORTB &= ~(_BV(PB4));
	if (takt>64) PORTB |= _BV(PB4);
	if ((takt++)==128) takt=0;
	}
}

int main(void)
{
/* Registerkofiguration */
DDRB=0xF1;								// PortB: 0,4,5,6,7 out; 1,2,3 in
DDRD=0xFF;								// PortD: all out
PORTB=0xF0;								// Display off, LED off, Switch off
PORTD=0xFF;								// all off
TIMSK =(_BV(TOIE0));					// enable TCNT0 overflow interrupt
TCCR0 =(_BV(CS00)+_BV(CS01));				// use CLK/64 prescale

bPortB.bPin = 1;						// Variable initialisieren
bPortB.bStatus = 0;
nCounter = 0;

cli();

_EEGET(maxzeit,0x00);					// lese gespeicherten Wert aus EEPROM
while bit_is_set(EECR, EEWE)			// warte bis EEPROM bereit 
	{
	}	
if (maxzeit>99) maxzeit=15;				// nur zur Sicherheit
zeit = maxzeit;

PORTB &= ~(_BV(PB0));					// sicherstellen dass Schaltausgang aus
sei();

while(1)
	{
	if (bit_is_clear (PINB, PINB3)) 
		{
		if (bPortB.bPin) 
			{
			nCounter = 0;				// Zhler initialisieren
			bPortB.bPin = 0;			// Aktuellen Portzustand speichern
			bPortB.bStatus = 0;			// Status als 'Nicht gedrckt' initialisieren
			}
			else if (!bPortB.bStatus)   // Nur wenn Status 'Nicht gedrckt'
				{
				if (++nCounter > DELAY_LOOPS) 
					{
					toggle_state();		// Betriebsmodues umschalten
					bPortB.bStatus = 1;	// Tastenstatus merken
					}
				}
		}
			else						// Taster ist nicht gedrckt
				{
				bPortB.bPin = 1;		// Aktuellen Zustand speichern
				}
	}
}



